from IPython.parallel import Client
import os, sys
from .gears import *

def centersearch(size,progo='picluster',clearprompt='n',algorithm='edge'):
	#like ipparasearch, but start from a clustered position
	#use center-seeking and/or edge-seeking algorithm

	#start the controller and view
	#progo=raw_input('Profile? ')
	c=Client(profile=progo)
	dview=c[:]
	dview.block=False

	#import on engines
	with dview.sync_imports():
		import numpy
		from math import sqrt, ceil
		import time, sys

	numworkers=len(c.ids)
	#size=int(raw_input('How large of an area? '))
	#clearprompt=raw_input('Clear the screen? y/n ')
	#algorithm=raw_input('Which algorithm? edge/center ')+"seek"
	gen='bishop'
	bishopdic=[0,2,5,7,1,3,4,6]
	rookdic=[0,1,2,3,4,5,6,7]
	dicname=gen+"dic"

	#define organizational means and variables
	if gen=='bishop':
		def genmyspotlist(myspot): #bishop bias (for central use only)
			myspot=list(myspot)
			myspotlist=[None]*8
			myspotlist[0]=(myspot[0]-1,myspot[1]-1)
			myspotlist[4]=(myspot[0],myspot[1]-1)
			myspotlist[1]=(myspot[0]+1,myspot[1]-1)
			myspotlist[5]=(myspot[0]-1,myspot[1])
			myspotlist[6]=(myspot[0]+1,myspot[1])
			myspotlist[2]=(myspot[0]-1,myspot[1]+1)
			myspotlist[7]=(myspot[0],myspot[1]+1)
			myspotlist[3]=(myspot[0]+1,myspot[1]+1)
			return myspotlist
	elif gen=='rook':
		def genmyspotlist(myspot): #rook bias (for central use only)
			myspot=list(myspot)
			myspotlist=[None]*8
			myspotlist[4]=(myspot[0]-1,myspot[1]-1)
			myspotlist[0]=(myspot[0],myspot[1]-1)
			myspotlist[5]=(myspot[0]+1,myspot[1]-1)
			myspotlist[1]=(myspot[0]-1,myspot[1])
			myspotlist[2]=(myspot[0]+1,myspot[1])
			myspotlist[6]=(myspot[0]-1,myspot[1]+1)
			myspotlist[3]=(myspot[0],myspot[1]+1)
			myspotlist[7]=(myspot[0]+1,myspot[1]+1)
			return myspotlist
	else:
		def genmyspotlist(myspot): #basic bias (for central use only)
			myspot=list(myspot)
			myspotlist=[None]*8
			myspotlist[0]=(myspot[0]-1,myspot[1]-1)
			myspotlist[1]=(myspot[0],myspot[1]-1)
			myspotlist[2]=(myspot[0]+1,myspot[1]-1)
			myspotlist[3]=(myspot[0]-1,myspot[1])
			myspotlist[4]=(myspot[0]+1,myspot[1])
			myspotlist[5]=(myspot[0]-1,myspot[1]+1)
			myspotlist[6]=(myspot[0],myspot[1]+1)
			myspotlist[7]=(myspot[0]+1,myspot[1]+1)
			return myspotlist
	def genmyspotlist0(myspot): #-1-1 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[0]=(myspot[0]-1,myspot[1]-1)
		myspotlist[1]=(myspot[0],myspot[1]-1)
		myspotlist[3]=(myspot[0]+1,myspot[1]-1)
		myspotlist[2]=(myspot[0]-1,myspot[1])
		myspotlist[5]=(myspot[0]+1,myspot[1])
		myspotlist[4]=(myspot[0]-1,myspot[1]+1)
		myspotlist[6]=(myspot[0],myspot[1]+1)
		myspotlist[7]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist7(myspot): #+1+1 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[7]=(myspot[0]-1,myspot[1]-1)
		myspotlist[6]=(myspot[0],myspot[1]-1)
		myspotlist[4]=(myspot[0]+1,myspot[1]-1)
		myspotlist[5]=(myspot[0]-1,myspot[1])
		myspotlist[2]=(myspot[0]+1,myspot[1])
		myspotlist[3]=(myspot[0]-1,myspot[1]+1)
		myspotlist[1]=(myspot[0],myspot[1]+1)
		myspotlist[0]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist5(myspot): #-1+1 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[3]=(myspot[0]-1,myspot[1]-1)
		myspotlist[5]=(myspot[0],myspot[1]-1)
		myspotlist[7]=(myspot[0]+1,myspot[1]-1)
		myspotlist[2]=(myspot[0]-1,myspot[1])
		myspotlist[6]=(myspot[0]+1,myspot[1])
		myspotlist[0]=(myspot[0]-1,myspot[1]+1)
		myspotlist[1]=(myspot[0],myspot[1]+1)
		myspotlist[4]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist2(myspot): #+1-1 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[4]=(myspot[0]-1,myspot[1]-1)
		myspotlist[1]=(myspot[0],myspot[1]-1)
		myspotlist[0]=(myspot[0]+1,myspot[1]-1)
		myspotlist[6]=(myspot[0]-1,myspot[1])
		myspotlist[2]=(myspot[0]+1,myspot[1])
		myspotlist[7]=(myspot[0]-1,myspot[1]+1)
		myspotlist[5]=(myspot[0],myspot[1]+1)
		myspotlist[3]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist1(myspot): #0-1 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[2]=(myspot[0]-1,myspot[1]-1)
		myspotlist[0]=(myspot[0],myspot[1]-1)
		myspotlist[1]=(myspot[0]+1,myspot[1]-1)
		myspotlist[3]=(myspot[0]-1,myspot[1])
		myspotlist[4]=(myspot[0]+1,myspot[1])
		myspotlist[6]=(myspot[0]-1,myspot[1]+1)
		myspotlist[7]=(myspot[0],myspot[1]+1)
		myspotlist[5]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist6(myspot): #0+1 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[5]=(myspot[0]-1,myspot[1]-1)
		myspotlist[7]=(myspot[0],myspot[1]-1)
		myspotlist[6]=(myspot[0]+1,myspot[1]-1)
		myspotlist[4]=(myspot[0]-1,myspot[1])
		myspotlist[3]=(myspot[0]+1,myspot[1])
		myspotlist[1]=(myspot[0]-1,myspot[1]+1)
		myspotlist[0]=(myspot[0],myspot[1]+1)
		myspotlist[2]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist4(myspot): #+10 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[6]=(myspot[0]-1,myspot[1]-1)
		myspotlist[3]=(myspot[0],myspot[1]-1)
		myspotlist[2]=(myspot[0]+1,myspot[1]-1)
		myspotlist[7]=(myspot[0]-1,myspot[1])
		myspotlist[0]=(myspot[0]+1,myspot[1])
		myspotlist[5]=(myspot[0]-1,myspot[1]+1)
		myspotlist[4]=(myspot[0],myspot[1]+1)
		myspotlist[1]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	def genmyspotlist3(myspot): #-10 biased. need to bias according to start
		myspot=list(myspot)
		myspotlist=[None]*8
		myspotlist[1]=(myspot[0]-1,myspot[1]-1)
		myspotlist[4]=(myspot[0],myspot[1]-1)
		myspotlist[5]=(myspot[0]+1,myspot[1]-1)
		myspotlist[0]=(myspot[0]-1,myspot[1])
		myspotlist[7]=(myspot[0]+1,myspot[1])
		myspotlist[2]=(myspot[0]-1,myspot[1]+1)
		myspotlist[3]=(myspot[0],myspot[1]+1)
		myspotlist[6]=(myspot[0]+1,myspot[1]+1)
		return myspotlist
	#dview['genmyspotlist']=genmyspotlist
	def createcenter(layer,midspot):
		midspot=list(midspot)
		layerspots=[None]*(8*(layer-1))
		def varyx(mult=1):
			thissection=[]
			for x in range(-1*(layer-1),layer+1):
				thissection.append(tuple([midspot[0]+mult*x,midspot[1]+mult*layer]))
			return thissection
		def varyy(mult=1):
			thissection=[]
			for y in range(-1*(layer-1),layer+1):
				thissection.append(tuple([midspot[0]+mult*layer,midspot[1]+-1*mult*y]))
			return thissection
		layerspots[0:2*(layer-1)-1]=varyx()
		layerspots[2*(layer-1):4*(layer-1)-1]=varyy()
		layerspots[4*(layer-1):6*(layer-1)-1]=varyx(mult=-1)
		layerspots[6*(layer-1):8*(layer-1)-1]=varyy(mult=-1)
		return layerspots
	def nearestzero(myspot,map):
		myspotlist=genmyspotlist(myspot)
		listofbadspots=[]
		for i in range(0,8):
			if len(map) in myspotlist[i] or -1 in myspotlist[i]:
				listofbadspots.append(myspotlist[i])
		for i in listofbadspots:
			myspotlist.remove(i)
		x,y=numpy.where(map==0)
		zerospots=zip(x,y)
		if len(zerospots)==0:
			print 'could not find any zeros, will stay put'
			return myspot
		zerodists=[None]*len(zerospots)
		for i in range(0,len(zerospots)):
			zerodists[i]=sqrt((myspot[0]-zerospots[i][0])**2+(myspot[1]-\
				   zerospots[i][1])**2)
		closestzero=zerospots[zerodists.index(min(zerodists))]
		distlist=[None]*len(myspotlist)
		for i in range(0,len(myspotlist)):
			distlist[i]=sqrt((closestzero[0]-myspotlist[i][0])**2+\
					 (closestzero[1]-myspotlist[i][1])**2)
		return tuple(myspotlist[distlist.index(min(distlist))])
	dview['nearestzero']=nearestzero
	background=numpy.zeros((size,size))
	spots=[None]*numworkers
	def updatemap():
		for spot in spots:
			background[spot[0],spot[1]]=background[spot[0],spot[1]]+1;

	#define different search methods and push to engines
	def purgelist(myspotlist,map,nope=False,best=False):
		if nope:
			if best:
				myspotlist.remove(nope)
			else:
				for i in range(len(nope)):
					if not len(nope[i])==2:
						aowrungoaew
					if nope[i] in myspotlist:
						myspotlist.remove(nope[i])
		listofbadspots=[]
		for i in range(0,len(myspotlist)):
			if len(map) in myspotlist[i] or -1 in myspotlist[i] or map[\
						 myspotlist[i][0],myspotlist[i][1]]!=0:
				listofbadspots.append(myspotlist[i])
		for i in listofbadspots:
			myspotlist.remove(i)
		return myspotlist
	dview["purgelist"]=purgelist
	def edgeseek(center,myspot,map,nope=False):
		if type(myspot[0])==tuple:
			myspot=myspot[0]
		oktogo=0
		repeats=0
		myspotlist=genmyspotlist(myspot)
		#if nope:
		#	if len(nope)>2:
		#		if nope[len(nope)-2]==nope[len(nope)-1]:
		#			nope=nope[len(nope)-1]
		myspotlist=purgelist(myspotlist,map,nope)
		if len(myspotlist)==0:
			return [nearestzero(myspot,map),0]
		#if len(myspotlist)==1:
		#	return list(tuple(myspotlist[0]),1)
		distlist=[None]*len(myspotlist)
		for i in range(len(distlist)):
			distlist[i]=sqrt((center[0]-myspotlist[i][0])**2+(center[1]-\
						myspotlist[i][1])**2)
		bestspot=tuple(myspotlist[distlist.index(max(distlist))])
		bestspotlist=purgelist(genmyspotlist(bestspot),map,myspot,1)
		locklist=[]
		needtoloop=len(bestspotlist)==1# and bestspotlist[0] in myspotlist
		oldbestspots=[]
		locklist=[]
		while needtoloop:
			if not len(locklist)==0:
				oldbestspots.append(deepbestspot)
			else:
				oldbestspots.append(myspot)
				oldbestspots.append(bestspot)
			deepbestspot=bestspotlist[0]
			locklist.append(deepbestspot)
			bestspotlist=purgelist(genmyspotlist(deepbestspot),map,oldbestspots)
			for spot in locklist:
				if spot in bestspotlist:
					bestspotlist.remove(spot)
			if len(bestspotlist)==1:
				deepbestspot=bestspotlist[0]
			else:
				needtoloop=0
			#return [bestspot,tuple(bestspotlist[0])]
		#if nope:
		#	if len(nope)==1:
		#		sys.stdout.write("something has gone terribly wrong"+str(nope))
		#		if list(bestspot)==list(nope[0]):
		#			aosinruaonergapojrg
		if len(locklist)>0:
			return [bestspot,tuple(locklist)]
		else:
			return [bestspot,0]
		#if len(myspotlist)==1:
		#	return [bestspot,myspotlist[0]]
		#return [bestspot,0]
	dview['edgeseek']=edgeseek
	def centerseek(center,myspot,map,nope=False):
		if type(myspot[0])==tuple:
			myspot=myspot[0]
		oktogo=0
		repeats=0
		#for now, redefining center to be center of map
	
		myspotlist=genmyspotlist(myspot)
		if nope:
			myspotlist.remove(nope)
		listofbadspots=[]
		for i in range(0,len(myspotlist)):
			if len(map) in myspotlist[i] or -1 in myspotlist[i] or map[\
						 myspotlist[i][0],myspotlist[i][1]]!=0:
				listofbadspots.append(myspotlist[i])
		for i in listofbadspots:
			myspotlist.remove(i)
		if len(myspotlist)==0:
			return nearestzero(myspot,map)
		#if len(myspotlist)==1:
		#	return list(tuple(myspotlist[0]),1)
		if len(myspotlist)==1:
			print "only one option"
			return tuple(myspotlist[0])
		distlist=[None]*len(myspotlist)
		for i in range(0,len(distlist)):
			distlist[i]=sqrt((center[0]-myspotlist[i][0])**2+(center[1]-\
						myspotlist[i][1])**2)
		return tuple(myspotlist[distlist.index(min(distlist))])
	#only difference between the 2 algorithms is min v max in the last line
	dview["centerseek"]=centerseek

	#initialize map w/ clustered drones
	midspot=tuple([size/2,size/2])
	midspots=genmyspotlist(midspot)
	background[midspot[0],midspot[1]]=1 #saying center of release is known
	if numworkers<=8:
		spots=midspots[:numworkers]
		for i in range(numworkers):
			exec 'alignbias='+dicname+'['+str(i)+']' in globals(), locals()
			exec 'c[c.ids[i]]["genmyspotlist"]=genmyspotlist'+str(alignbias) in globals(), locals()
			exec 'print str(c.ids[i])+" is algorithm "+str(alignbias)' in globals(), locals()
	else:
		for layer in range(2,(int(sqrt(numworkers))+1)/2+2):
			thislayerend=min(8*(layer-1),numworkers)
			thislayerstart=8*(layer-2)
			spots[thislayerstart:thislayerend]=createcenter(layer,midspot)[:thislayerend-thislayerstart]
	for worker in spots:
		print "worker at "+str(worker)
	updatemap()
	oldspots=[]
	oldspots.append(tuple(spots))
	iteration=1
	alert=0

	#start the search
	if clearprompt=='y':
		os.system('clear')
	print 'starting the search'
	print 'iteration '+str(iteration)
	print background

	while 0 in background:
		dview=c[:]
		if iteration==1:
			print 'starting at '+str(spots)
		#else:
		#	print str(oldspots[iteration-2])+' >>> '+str(spots)
		center=[None,None]
		center[0]=sum([x for x,y in spots])/len(spots)
		center[1]=sum([y for x,y in spots])/len(spots)
		starttime=time.time()
		dview.scatter('myspot',spots)
		dview['center']=center
		if algorithm=='centerseek':
			dview['center']=midspot
		dview['map']=background
		dview.execute('[newspot,mylocks]='+algorithm+'(center,myspot,map)')
		locklist=[]
		for i in c.ids:
			thislocks=c[i]["mylocks"]
			if thislocks:
				#print thislocks
				for j in thislocks:
					locklist.append(j)
		dview["locklist"]=locklist
		#print locklist
		for i in c.ids: #or use dview.gather, but duplicates
			intermid=c[i]['newspot']
			lockviolation=0
			if intermid in locklist:
				lockviolation=1
				#print "attempt to move into lock by "+str(i)
			if (not intermid in spots) and not lockviolation:
				spots[c.ids.index(i)]=intermid
			else:
				#print 'possible problem with '+str(i)
				#if crashtest==3:
				#	sys.exit('drone '+str(i)+' stalled')
				c[i].execute("locklist.append(newspot)")
				c[i].execute('[newspot,mylocks]='+algorithm+'(center,myspot,map,locklist)')
				intermid=c[i]['newspot']
				if intermid in spots:
					#print "possible meta problem with "+str(c.ids[spots.index(intermid)])
					spots[c.ids.index(i)]=intermid
					c[c.ids[spots.index(intermid)]].execute("locklist.append(newspot)")
					c[c.ids[spots.index(intermid)]].execute('[newspot,mylocks]='+algorithm+'(center,myspot,map,locklist)')
					spots[spots.index(intermid)]=c[c.ids[spots.index(intermid)]]["newspot"]
				#if spots[i]==oldspots[iteration-1][i]:
				#	crashtest=crashtest+1
					#print str(i)+"/"+str(intermid)+" got here, with other locklist "+str(c[c.ids[spots.index(intermid)]]["mylocks"])
					if intermid in spots:
						#print locklist
						#c[i].execute("locklist=[newspot,]")
						c[i]["locklist"]=[intermid,]
						#print c[i]["locklist"]
						c[i].execute('[newspot,mylocks]='+algorithm+'(center,myspot,map,locklist)')
						intermid=c[i]['newspot']
						#print intermid
						#print "possible contradiction problem here with "+str(i)
						spots[c.ids.index(i)]=intermid
				else:
					spots[c.ids.index(i)]=intermid
		updatemap()
		oldspots.append(tuple(spots))
		iteration=iteration+1
		if clearprompt=='y':
			os.system('clear')
		print 'iteration '+str(iteration)
		print background
		print 'percent done: '+str(float((background>0).sum())/(size**2)*100)+'%'
		print str(oldspots[iteration-2])+' >>> '+str(spots)
		if (background>1).sum()>0 and alert==0:
			if raw_input("imperfect search. continue? y/n ")=='n':
				sys.exit("exiting")
			else:
				alert=1

	#once done, print the final report
	print 'final report: '
	print "moves wasted: (doesn't account for necessary moves) "+str(-size**2+numworkers*iteration+1)
	print 'workers: '+str(numworkers)
	print 'number of squares: '+str(size**2)
	print 'extra iterations: '+str(iteration-ceil(size**2/numworkers))
	print 'percent error in terms of iterations: '+str((iteration-\
					ceil(size**2/numworkers\
					))/iteration*100)+'%'
	if alert==1:
		print "imperfect"
	else:
		print "perfect"


import cv2, time, sys, cPickle, zmq, os, copy
from math import log
from IPython.parallel import Client
from scipy import stats
import numpy as np


def findfaceswithtrigger(profile='picluster',threshold=15,n=100,serial='no',show=False):
	import sys, cPickle, os
	pidfile=open('mypythonpid','w')
	cPickle.dump(os.getpid(),pidfile)
	pidfile.close()
	runstate=open('runstate','w')
	cPickle.dump(0,runstate)
	runstate.close()
	print os.getpid()
	#context=zmq.Context()
	#pubsocket=context.socket(zmq.PUB)
	#port=5556
	#pubsocket.bind("tcp://*:%s"%port)
	#subsocket=context.socket(zmq.SUB)
	if sys.platform=='darwin':
		ips=open("/Users/isaac/Code/parastuff/ipaddresses").read()
	if sys.platform=='linux2':
		ips=open("/home/pi/ipaddresses").read()
	splitips=ips.split("\n")
	print splitips
	#for ip in splitips:
	#	if ip:
	#		subsocket.connect("tcp://%s:%s"%(ip,port))
	#subsocket.setsockopt(zmq.SUBSCRIBE,"need quiet?")

	c=Client(profile=profile)
	dview=c[:]
	numnodes=len(c.ids)
	dview.block=False
	with dview.sync_imports():
		import cv2,sys,time,math,os,cPickle,signal
	#dview.execute("mypid=os.getpid()")
	def getmypid():
		import cPickle
		try:
			file=open('mypythonpid','r')
			mypid=cPickle.load(file)
			file.close()
		except:
			mypid=False
		return mypid
	def getrunstate():
		import cPickle
		try:
			file=open('runstate','r')
			state=cPickle.load(file)
			file.close()
		except:
			state=0
		return state
	dview["getrunstate"]=getrunstate
	dview["getmypid"]=getmypid
	dview.execute("mypid=getmypid()")
	for i in range(numnodes):
			print c[c.ids[i]]['mypid']
	print sys.platform
	if sys.platform=='darwin':
		facedetector=cv2.CascadeClassifier("/usr/local/Cellar/opencv/2.4.9/share/OpenCV/haarcascades/haarcascade_frontalface_alt2.xml")
	elif sys.platform=='linux2':
		facedetector=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")
	else: 
		print "something went wrong detecting the system"
	times=gen.loadtimes()
	scale=1.1
	dview["scale"]=scale
	for i in c.ids:
		c[i].execute('if sys.platform=="darwin": haarface=cv2.CascadeClassifier("/usr/local/Cellar/opencv/2.4.9/share/OpenCV/haarcascades/haarcascade_frontalface_alt2.xml")')
		c[i].execute('if sys.platform=="linux2": haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")')
	
	numsizes=len(times)
	trans=[None]*numsizes
	for i in range(numsizes):
		trans[i]=int(20*(scale**i))
	
	mintime=min(times)
	times=[thistime-mintime for thistime in times]

	print "predicting even distribution..."
	def predictdist(numnodes,times,numsizes,c):
		timepernode=sum(times)/numnodes
		thresholds=[None]*(numnodes+1)
		thresholds[0]=0
		for i in range(1,numnodes+1):
			j=thresholds[i-1]+1
			while sum(times[thresholds[i-1]:j])<timepernode and j<numsizes:
				j=j+1
				#print j
				#time.sleep(3)
			if sum(times[thresholds[i-1]:j])-timepernode>timepernode-sum(times[thresholds[i-1]:j-1]):
				j=j-1 #makes more even, but last few take longest...otherwise even but last is really short
			thresholds[i]=j
		thresholds[numnodes]=numsizes-1
		for i in range(numnodes):
			c[c.ids[i]]['thismin']=tuple([trans[thresholds[i]]-1]*2)
			c[c.ids[i]]['thismax']=tuple([trans[thresholds[i+1]]+1]*2)
	predictdist(numnodes,times,numsizes,c)
	
	def findfaces(img,scale,serial):
		starttime=time.time()
		numnodes=len(c.ids)
		img=cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
		dview['img']=img
		dview.execute('img=img.copy()')
		dview.execute("mypid=getmypid()")
		runstate=open('runstate','w')
		cPickle.dump(1,runstate)
		runstate.close()
		dview.execute("state=getrunstate()")
		#for i in range(numnodes):
		#	print c[c.ids[i]]["state"]
		dview.execute("if mypid and not state: os.kill(mypid,signal.SIGTSTP)")
		dview.execute("exectime=time.time();myfaces=haarface.detectMultiScale(img,scale,4,1,thismin,thismax);exectime=time.time()-exectime;endtime=time.time()",block=False)
		faces=set()
		thistimes=[None]*numnodes
		thistimesremote=[None]*numnodes
		for i in range(numnodes):
			thistimes[i]=time.time()
			print "checking engine "+str(c.ids[i])
			thisfaces=c[c.ids[i]]["myfaces"]
			thistimesremote[i]=c[c.ids[i]]["exectime"]
			print len(thisfaces)
			if len(thisfaces)>0:
				for face in thisfaces:
					faces.add(tuple(face))
			thistimes[i]=time.time()-thistimes[i]
		print thistimes
		print thistimesremote

		duptime=time.time()
		facelist=list(faces)
		donttest=[]
		finallist=[]
		testlen=len(facelist)
		origlist=facelist
		testlennew=0
		print len(origlist)
		while not testlennew==testlen:
			testlen=len(facelist)
			for first in range(len(facelist)):
				if first not in donttest:
					for second in range(first+1,len(facelist)):
						sub=facelist[first]
						j=facelist[second]
						var=abs(j[3]-sub[3])/2
						if ((j[0]+j[3]/2-var<sub[0]+sub[3]/2) and (j[0]+j[3]/2+var>sub[0]+sub[3]/2)) and ((j[1]+j[2]/2-var<sub[1]+sub[2]/2) and (j[1]+j[2]/2+var>sub[1]+sub[2]/2)) and min([float(j[3])/sub[3],float(sub[3])/j[3]])>1.0/2.0:
							donttest.append(first)
							donttest.append(second)
							#facelist.append(tuple([(j[0]+sub[0])/2,(j[1]+sub[1])/2,(j[2]+sub[2])/2,(j[3]+sub[3])/2]))
							inter=tuple([(j[0]+sub[0])/2,(j[1]+sub[1])/2,(j[2]+sub[2])/2,(j[3]+sub[3])/2])
							finallist.append(inter)
							break
					if first not in donttest:
						finallist.append(facelist[first])
			testlennew=len(finallist)
			facelist=finallist
			finallist=[]
			donttest=[]
		finallist=facelist
		duptime=time.time()-duptime
		print len(finallist)
		starttime=time.time()-starttime
		print "min/avg/max/regtime/tottime: "+str(min(thistimesremote))+"/"+str(sum(thistimesremote)/numnodes)+"/"+str(max(thistimesremote))+"/"+str(starttime)+"/"+str(sum(thistimesremote))

		if serial=='yes':
			serialtime=time.time()
			facedetector.detectMultiScale(img,scale,4,1)
			serialtime=time.time()-serialtime
			print serialtime
		
		dview.execute("if mypid: os.kill(mypid,signal.SIGCONT)")
		#for i in range(numnodes):
		#	print c[c.ids[i]]['mypid']
		runstate=open('runstate','w')
		cPickle.dump(0,runstate)
		runstate.close()
		return finallist
	
	def calibcam(n,cam):
		sumlist=[None]*n
		for i in range(n):
			retval,img=cam.read()
			xstart=img.shape[0]/4
			ystart=img.shape[1]/4
			sumlist[i]=img[xstart:(xstart*3),ystart:(ystart*3),:].sum()
		return sumlist
	
	cam=cv2.VideoCapture(0)
	
	retval,img=cam.read()
	gen.dumptimes(img,scale=1.1)
	
	sumlist=calibcam(n,cam)
	std=stats.tstd(sumlist)
	avg=sum(sumlist)/n
	framenum=0
	oldids=c.ids
	
	print "starting detection"
	
	while True:
		retval,img=cam.read()
		xstart=img.shape[0]/4
		ystart=img.shape[1]/4
		thisz=(img[xstart:(xstart*3),ystart:(ystart*3),:].sum()-avg)/std
		#while True:
		#	print "testing for sleep needed"
		#	try:
		#		if subsocket.recv(flags=zmq.NOBLOCK)=="need quite? yes":
		#			print "need sleep"
		#			while True:
		#				try:
		#					if subsocket.recv(flags=zmq.NOBLOCK)=="need quite? not anymore":
		#						break
		#				except:
		#					pass
		#				time.sleep(.5)
		#	except:
		#		break
		if abs(thisz)>threshold:
			print "something weird, zscore="+str(thisz)
			time.sleep(.5)
		#	pubsocket.send("need quiet? yes")
			retval,newimg=cam.read() #works well on mac, maybe not pis...
			#if not len(c.ids)==len(oldids): #done periodically, not ideal to update here...
			#	print "cluster changed"
			#	numnodes=len(c.ids)
			#	dview=c[:]
			#	print "repredicting even distribution..."
			#	predictdist(numnodes,times,numsizes,c)
			#	for id in c.ids:
			#		if id not in oldids:
			#			c[id].execute('import cv2,sys,time,math,os,cPickle,signal;myfaces=[]')
			#			c[id]['getrunstate']=getrunstate
			#			c[id]['getmypid']=getmypid
			#			c[id]['scale']=scale
			#			c[id].execute('haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")')
					#else:
					#	c[id].execute('myfaces=[]')
			#	oldids=c.ids
			print "searching for faces..."
			faces=findfaces(newimg,scale,serial)
		#	pubsocket.send("need quiet? not anymore")
			if len(faces)>0:
				print "FOUND A FACEZ!!!!!11!"
				if show==True:
					for (x,y,h,w) in faces:
						cv2.rectangle(newimg,(x,y),(x+w,y+h),(0,255,255),1)
					#cv2.imshow("obj num "+str(i),newimg)
					cv2.imshow("obj found",newimg)
			else:
				print "no facez :("
				if show==True:
					cv2.imshow("obj not found",newimg)
			if show==True:
				cv2.waitKey(1)
			sumlist[0]=img[xstart:(xstart*3),ystart:(ystart*3),:].sum()
			std=stats.tstd(sumlist)
			avg=sum(sumlist)/n
		#else:
		#	pubsocket.send("need quiet? no")
		framenum=framenum+1
		if framenum%10==0:
			sumlist[0]=img[xstart:(xstart*3),ystart:(ystart*3),:].sum()
			std=stats.tstd(sumlist)
			avg=sum(sumlist)/n
		#if framenum%100==0:
		#	print framenum
		if framenum%(10*numnodes)==0:
			if not len(c.ids)==len(oldids):
				print "cluster changed"
				numnodes=len(c.ids)
				dview=c[:]
				print "repredicting even distribution..."
				predictdist(numnodes,times,numsizes,c)
				for id in c.ids:
					if id not in oldids:
						c[id].execute('import cv2,sys,time,math,os,cPickle,signal;myfaces=[]')
						c[id]['getrunstate']=getrunstate
						c[id]['getmypid']=getmypid
						c[id]['scale']=scale
						c[id].execute('haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")')
					#else:
					#	c[id].execute('myfaces=[]')
				oldids=c.ids
				#c.spin()
				#trying reseting the client
				c=Client(profile=profile)
				dview=c[:]


def getalignface(facefinder,eyefinder,nosefinder,cam):
	cv2.destroyAllWindows()
	print "finding faces..."
	retval,img=cam.read()
	if not retval:
		print "could not read image"
	cv2.imshow("img",img)
	#cv2.imshow("img",img)
	dims=facefinder.detectMultiScale(img,1.1,5,1,(20,20),img.shape[:2])
	print dims
	if len(dims)>0:
		sizelist=[w for (x,y,h,w) in dims]
		maxsize=max(sizelist)
		biggestface=[[x,y,h,w] for (x,y,h,w) in dims if w==maxsize and w>60]
		print biggestface
		if len(biggestface)>0:
			print "finding facial features..."
			for (x,y,h,w) in biggestface:
				thisface=img[x:(x+w),y:(y+h),:]
				cleanface=copy.deepcopy(thisface)
			cv2.imshow("thisface",thisface)
			eyes=eyefinder.detectMultiScale(thisface,1.1,30,1)
			noses=nosefinder.detectMultiScale(thisface,1.1,20,1)
			print "eyes: "+str(eyes)
			print "noses: "+str(noses)
			if len(noses)>0 and len(eyes)>1:
				maxnosesize=max([w for (x,y,h,w) in noses])
				biggestnose=[[x,y,h,w] for (x,y,h,w) in noses if w==maxnosesize][0]
				finaleyes=[]
				print "filtering out nostrils from the eyes..."
				for eye in eyes:
					if not (eye[0]>(biggestnose[0]-5) and eye[0]<(biggestnose[0]+biggestnose[3]+5) and eye[1]>(biggestnose[1]-5) and eye[1]<(biggestnose[1]+biggestnose[2]+5)):
						finaleyes.append(eye)
				print "eyes: "+str(finaleyes)
				print "noses: "+str(noses)
				for (x,y,h,w) in finaleyes:
					cv2.rectangle(thisface,(x,y),(x+w,y+h),(0,255,255),1)
				for (x,y,h,w) in [biggestnose]:
					cv2.rectangle(thisface,(x,y),(x+w,y+h),(255,255,0),1)
				#print biggestnose
			elif len(eyes)>1 and len(eyes)<3:
				print "could not find a nose, found 2 eyes. working with them"
				for (x,y,h,w) in eyes:
					cv2.rectangle(thisface,(x,y),(x+w,y+h),(0,255,255),1)
				cv2.imshow("thisface",thisface)
				finaleyes=eyes
			else: #add parsing for more than 2 detected eyes without a nose
				print "feature detection went wrong"
				return []
			cv2.imshow("thisface features",thisface)
			eye1mid=(finaleyes[0][0]+finaleyes[0][3]/2,finaleyes[0][1]+finaleyes[0][2]/2)
			try:
				eye2mid=(finaleyes[1][0]+finaleyes[1][3]/2,finaleyes[1][1]+finaleyes[1][2]/2)
			except:
				print "something went wrong with the format of eyes, skipping to next iteration"
				return []
			if eye1mid[0]<eye2mid[0]:
				lefteye=eye1mid
				righteye=eye2mid
			else:
				lefteye=eye2mid
				righteye=eye1mid
			borderoffset=9.0/12.0
			midborderoffset=3
			eyetriangle=[righteye[0]-lefteye[0],righteye[1]-lefteye[1]]
			topbordermid=[lefteye[0]+eyetriangle[0]/2+eyetriangle[1]/midborderoffset,lefteye[1]+eyetriangle[1]/2-eyetriangle[0]/midborderoffset]
			topborder=[(topbordermid[0]-int(eyetriangle[0]*borderoffset),topbordermid[1]-int(eyetriangle[1]*borderoffset)),(topbordermid[0]+int(eyetriangle[0]*borderoffset),topbordermid[1]+int(eyetriangle[1]*borderoffset))]
			bordertriangle=[topborder[1][0]-topborder[0][0],topborder[1][1]-topborder[0][1]]
			borderlength=int(np.sqrt(bordertriangle[0]**2+bordertriangle[1]**2))
			print bordertriangle
			border=[topborder[0],topborder[1],(topborder[1][0]-bordertriangle[1],topborder[1][1]+bordertriangle[0]),(topborder[0][0]-bordertriangle[1],topborder[0][1]+bordertriangle[0])]
			violation=0
			for i in range(4):
				for j in range(2):
					if border[i][j]>thisface.shape[0] or border[i][j]<0:
						violation=1			
			if violation:
				print "could not align face, features too close to edge"
			else:
				print "aligning face"
				cv2.line(thisface,eye1mid,eye2mid,(255,0,0),1)
				cv2.line(thisface,border[0],border[1],(0,255,0),1)
				cv2.line(thisface,border[2],border[3],(0,255,0),1)
				cv2.line(thisface,border[0],border[3],(0,255,0),1)
				cv2.line(thisface,border[1],border[2],(0,255,0),1)
				#cv2.line(thisface,(nosemid[0]+eyedist,nosemid[1]+noseedgeoffset),(nosemid[0]-eyedist,nosemid[1]+noseedgeoffset),(0,255,0),1)
				facecenter=tuple(np.array([border[0][0]+border[2][0],border[0][1]+border[2][1]])/2)
				cv2.circle(thisface,facecenter,5,(255,0,0),-1)
				cv2.imshow("thisface border",thisface)
				rotationangle=np.arctan(float(eyetriangle[1])/float(eyetriangle[0]))
				rotationmat=cv2.getRotationMatrix2D(facecenter,rotationangle*180/np.pi,1.0)
				print border[0]
				bordercenteroffset=[border[0][0]-facecenter[0],border[0][1]-facecenter[1]]
				print bordercenteroffset
				borderoffsetafterrotation=[bordercenteroffset[0]*np.cos(rotationangle)+bordercenteroffset[1]*np.sin(rotationangle),bordercenteroffset[0]*np.sin(rotationangle)-bordercenteroffset[1]*np.cos(rotationangle)-borderlength]
				print borderoffsetafterrotation
				borderafterrotation=(int(borderoffsetafterrotation[0]+facecenter[0]),int(borderoffsetafterrotation[1]+facecenter[1]))
				print borderafterrotation
				finalface=cv2.warpAffine(thisface,rotationmat,thisface.shape[:2],flags=cv2.INTER_LINEAR)
				finalcroppedface=finalface[borderafterrotation[1]:borderlength+borderafterrotation[1],borderafterrotation[0]:borderafterrotation[0]+borderlength]
				cv2.circle(finalface,borderafterrotation,3,(0,255,0),-1)
				#cv2.circle(finalface,(borderafterrotation[0],borderafterrotation[1]-borderlength),3,(0,255,0),-1)
				cv2.circle(finalface,facecenter,4,(0,255,255),-1)
				cv2.imshow("rotated",finalface)
				cv2.imshow("final cropped face",finalcroppedface)
				cleanface=cv2.warpAffine(cleanface,rotationmat,thisface.shape[:2],flags=cv2.INTER_LINEAR)[borderafterrotation[1]:borderlength+borderafterrotation[1],borderafterrotation[0]:borderafterrotation[0]+borderlength]
				cv2.imshow("clean face",cleanface)
				cleanface=cv2.cvtColor(cleanface,cv2.COLOR_BGR2GRAY)
				cleanface=cv2.resize(cleanface,(60,60))
				cv2.imshow("small clean face",cleanface)
				return cleanface
			while 1:
				k=cv2.waitKey(10)
				if k==27:
					break
				else:
					continue
	return []


def makepredictor():
	imgset=[]
	labels=[]
	facefinder=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")
	eyefinder=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_eye.xml")
	nosefinder=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_mcs_nose.xml")
	cam=cv2.VideoCapture(-1)
	while not raw_input("capture image? y/n ")=='n':
		face=getalignface(facefinder,eyefinder,nosefinder,cam)
		if len(face)>0:
			if not raw_input("is this the correct face? y/n ")=='n':
				imgset.append(face)
				labels.append(int(raw_input("who is this? (int) ")))

	labels=np.array(labels)
	facerec=cv2.createLBPHFaceRecognizer()
	facerec.train(imgset,labels)
	return facerec